
import bluej.extensions.*;
import bluej.extensions.event.*;
import bluej.extensions.editor.*;

import java.net.URL;
import javax.swing.*;
import java.awt.event.*;

/**
 * This extension help to:
 * 1) Generate source code of GET/SET methods of all class's properties
 * 2) Generate source code of GET method of one propertie
 * 3) Generate source code of SET method of one propertie
 * 4) Generate GUI javax.swing to view/edit object's properties with GET/SET methods
 * 
 * @author (Milton Jes&uacute;s Vera Contreras - miltonjesusvc@ufps.edu.co) 
 * @version 0.0000000000000001 --> Math.sin(Math.PI-Double.MIN_VALUE) --> :)
 */
public class GetterSetterExtension extends Extension
{
    private BPackage curPackage;
    private BClass curClass;
    private BObject curObject;

    /*
     * When this method is called, the extension may start its work.
     */
    public void startup (BlueJ bluej) {
        // Register a generator for menu items
        bluej.setMenuGenerator(new MenuBuilder(this));
    }

    /*
     * This method must decide if this Extension is compatible with the 
     * current release of the BlueJ Extensions API
     */
    public boolean isCompatible () { 
        return true; 
    }

    /*
     * Returns the version number of this extension
     */
    public String  getVersion () { 
        return ("0.0000000000000001 --> Math.sin(Math.PI-Double.MIN_VALUE) --> :)");  
    }

    /*
     * Returns the user-visible name of this extension
     */
    public String  getName () { 
        return ("GetterSetter Extension");  
    }

    public void terminate() {
        System.out.println ("GetterSetter Extension terminates");
    }

    public String getDescription () {
        return ("GetterSetter Extension");
    }

    /**
     * Returns a URL where you can find info on this extension.
     * The real problem is making sure that the link will still be alive in three years...
     */
    public URL getURL () {
        try {
            return new URL("http://www.bluej.org/doc/writingextensions.html");
        } catch ( Exception eee ) {
            // The link is either dead or otherwise unreachable
            System.out.println ("GetterSetterExtension: getURL: Exception="+eee.getMessage());
            return null;
        }
    }

    /*Only to Manifiest*/
    public static void main(String [] args)
    {
    }//end main

    /**GET Method propertie curPackage*/
    public BPackage getCurPackage(){
        return this.curPackage;
    }//end method getCurPackage

    /**SET Method propertie  curPackage*/
    public void setCurPackage(BPackage curPackage){
        this.curPackage = curPackage;
    }//end method setCurPackage

    /**GET Method propertie  curClass*/
    public BClass getCurClass(){
        return this.curClass;
    }//end method getCurClass

    /**SET Method propertie  curClass*/
    public void setCurClass(BClass curClass){
        this.curClass = curClass;
    }//end method setCurClass

    /**GET Method propertie  curObject*/
    public BObject getCurObject(){
        return this.curObject;
    }//end method getCurObject

    /**SET Method propertie  curObject*/
    public void setCurObject(BObject curObject){
        this.curObject = curObject;
    }//end method setCurObject

    /**
     * 1) Generate source code of GET/SET methods of all class's properties
     */ 
    public void genGetterSetter() throws Exception {
        EditorUtil editorUtil;
        CodeGenerator codeGenerator = null;
        String code ="";
        if(curClass.isCompiled()){
            editorUtil = new EditorUtil(this.curClass);
            codeGenerator = new CodeGenerator();
            code = codeGenerator.generateGetterSetter(this.curClass.getJavaClass());
            editorUtil.appendText(code);
            curClass.compile(false);
        }
        else
            throw new Exception("Class no compiled");
    }//fin genGetterSetter

    /**
     * 2) Generate source code of GET method of one propertie
     */
    public void genGetter(String propertie) throws Exception {
        EditorUtil editorUtil;
        CodeGenerator codeGenerator = null;
        String code ="";
        if(curClass.isCompiled()){
            editorUtil = new EditorUtil(this.curClass);
            codeGenerator = new CodeGenerator();
            code = codeGenerator.generateGetter(this.curClass.getJavaClass(), propertie);
            editorUtil.appendText(code);
            curClass.compile(false);
        }
        else
            throw new Exception("Class no compiled");
    }//fin genGetter

    /**
     *  3) Generate source code of SET method of one propertie
     */
    public void genSetter(String propertie) throws Exception {
        EditorUtil editorUtil;
        CodeGenerator codeGenerator = null;
        String code ="";
        if(curClass.isCompiled()){
            editorUtil = new EditorUtil(this.curClass);
            codeGenerator = new CodeGenerator();
            code = codeGenerator.generateSetter(this.curClass.getJavaClass(), propertie);
            editorUtil.appendText(code);
            curClass.compile(false);
        }
        else
            throw new Exception("Class no compiled");
    }//fin genSetter    

    /**
     * 4) Generate GUI javax.swing to view/edit object's properties with GET/SET methods
     */
    public void setterObject() throws Exception
    {
        GetterSetterFrameObject gsfo = new GetterSetterFrameObject(this.curObject);
    }//fin setterObject

}//fin class GetterSetterExtension